#!/usr/bin/env node
"use strict";

var fs = require('fs');

var path = require('path');

var meow = require('meow');

var unique = require('array-unique');

var request = require('request');

var userhome = require('user-home');

var spinner = require('ora')();

var cli = meow(["\n    Usage:\n\n      Uploading images\n        imgur-upload path/to/image.jpg\n        imgur-upload path/to/image-one.jpg path/to/image-two.jpg\n        imgur-upload path/to/*.jpg\n\n      Uploading latest image in a directory\n        imgur-upload latest path/to/directory\n\n      Setting default image in a directory\n        imgur-upload basedir path/to/directory\n        imgur-upload latest\n\n      Viewing upload history\n        imgur-upload history\n\n      Clear upload history\n        imgur-upload clear\n\n    Options:\n\n      --delete, -d     Delete image file after being uploaded\n"], {
  alias: {
    'd': 'delete'
  },
  boolean: ['delete']
});
/** normalize command and parameter values **/

switch (cli.input[0]) {
  case 'latest':
  case 'basedir':
    var command = cli.input[0];
    var parameter = cli.input[1];
    break;

  case 'history':
  case 'clear':
    var command = cli.input[0];
    break;

  default:
    var command = 'upload';
    var parameters = unique(cli.input);
}

var flags = cli.flags;
/** execute commands **/

if (command === 'upload' && parameters.length === 1) {
  spinner.start();
  spinner.text = "Uploading ".concat(parameters[0]);
  uploadImage(parameters[0], null, function (link) {
    spinner.stop();
    addUploadHistoryItem(link);

    if (flags.delete) {
      deleteImage(parameters[0]);
    }

    console.log(link);
  });
} else if (command === 'upload' && parameters.length > 1) {
  spinner.start();
  uploadAlbum(parameters, function (link) {
    spinner.stop();
    addUploadHistoryItem(link);
    console.log(link);
  });
} else if (command === 'history') {
  var history = moduleConfig().history;

  if (history) {
    history.forEach(function (link) {
      console.log(link);
    });
  }
} else if (command === 'clear') {
  clearUploadHistory();
  console.log('imgur-upload history cleared');
} else if (command === 'basedir' && parameter) {
  setBaseDir(parameter);
} else if (command === 'basedir' && !parameter) {
  try {
    console.log(getBaseDir());
  } catch (e) {
    console.log(e.message);
  }
} else if (command === 'latest' && parameter) {
  spinner.start();
  uploadLatestImageInDirectory(parameter, function (link) {
    spinner.stop();
    addUploadHistoryItem(link);
    console.log(link);
  });
} else if (command === 'latest' && !parameter) {
  try {
    var basedir = getBaseDir();
    spinner.start();
    uploadLatestImageInDirectory(basedir, function (link) {
      spinner.stop();
      addUploadHistoryItem(link);
      console.log(link);
    });
  } catch (e) {
    console.log(e.message);
  }
}

function uploadImage(image, album, callback) {
  var data = {
    url: 'https://api.imgur.com/3/image',
    method: 'POST',
    formData: {
      image: fs.createReadStream(image)
    }
  };

  if (album) {
    data.formData.album = album;
  }

  sendApiRequest(data, function (response) {
    callback("http://imgur.com/".concat(response.data.id));
  });
}

function uploadAlbum(images, callback) {
  spinner.text = 'Creating album';
  var data = {
    url: 'https://api.imgur.com/3/album',
    method: 'POST'
  };
  sendApiRequest(data, function (response) {
    var albumid = response.data.deletehash;
    var index = 0;

    (function uploadAlbumImage() {
      if (index < images.length) {
        var image = images[index++];
        spinner.text = "Uploading ".concat(image);
        uploadImage(image, albumid, function () {
          if (flags.delete) {
            deleteImage(image);
          }

          uploadAlbumImage();
        });
      } else {
        callback("http://imgur.com/a/".concat(response.data.id));
      }
    })();
  });
}

function uploadLatestImageInDirectory(directory, callback) {
  fs.readdir(directory, function (error, files) {
    var images = files.filter(function (file) {
      return /\.(jpg|jpeg|png|gif|bmp)$/.test(file);
    });
    var latestImage = images.reduce(function (latest, current) {
      var latestStat = fs.statSync(path.join(directory, latest));
      var currentStat = fs.statSync(path.join(directory, current));
      var latestModifiedTime = new Date(latestStat.mtime);
      var currentModifiedTime = new Date(currentStat.mtime);

      if (latestModifiedTime.valueOf() > currentModifiedTime.valueOf()) {
        return latest;
      }

      return current;
    }, images[0]);
    var latestImagePath = path.join(directory, latestImage);
    spinner.text = "Uploading ".concat(latestImagePath);
    uploadImage(latestImagePath, null, function (link) {
      if (flags.delete) {
        deleteImage(latestImagePath);
      }

      callback(link);
    });
  });
}

function sendApiRequest(data, callback) {
  data.headers = data.headers || {};
  data.headers['Authorization'] = 'Client-ID ae51d45d93313f1';
  request(data, function (error, response, body) {
    if (error) {
      throw error;
    }

    callback(JSON.parse(body));
  });
}

function deleteImage(image) {
  fs.unlinkSync(image);
}

function setBaseDir(basedir) {
  var config = moduleConfig();
  config.basedir = basedir;
  moduleConfig(config);
}

function getBaseDir(basedir) {
  var config = moduleConfig();

  if (config.basedir) {
    return config.basedir;
  }

  throw new Error('imgur-upload basedir is not yet set');
}

function addUploadHistoryItem(link) {
  var config = moduleConfig();

  if (!('history' in config)) {
    config.history = [];
  }

  config.history.push(link);
  moduleConfig(config);
}

function clearUploadHistory() {
  var config = moduleConfig();
  config.history = [];
  moduleConfig(config);
}

function moduleConfig(config) {
  var configDirectory = path.join(userhome, '.imgur-upload');

  try {
    fs.statSync(configDirectory);
  } catch (e) {
    fs.mkdirSync(configDirectory);
  }

  var configFile = path.join(configDirectory, 'config.json');

  try {
    fs.statSync(configFile);
  } catch (e) {
    fs.writeFileSync(configFile, '{}');
  }

  if (config) {
    return fs.writeFileSync(configFile, JSON.stringify(config));
  }

  var contents = fs.readFileSync(configFile);
  return JSON.parse(contents);
}